<?php
/*
    Description:
        1. Various function utilities created to support the StripeFPX module
        2. Includes functions to parse WHMCS parameters and format them into request parameters passed to a Stripe API call

    Last Updated: 3-JUL-21
*/

namespace StripeFPX;

class Utility
{
    public static function getApiKeys($whmcs)
    {
        if ( empty( $whmcs['publishable_key'] ) || empty( $whmcs['secret_key'] ) ) {
            return false;
        }

        return true;
    }

    public static function moduleParser($whmcs)
    {
        $module = array();
        $module['name'] = $whmcs['name'];
        $module['publishable_key'] = $whmcs['publishable_key'];
        $module['secret_key'] = $whmcs['secret_key'];
        $module['call_to_action'] = $whmcs['langpaynow'];

        return $module;
    }

    public static function clientParser($whmcs)
    {
        $client = array();
        $client['url'] = $whmcs['systemurl'] . 'admin/clientssummary.php?userid=' . $whmcs['clientdetails']['userid'];
        $client['email'] = $whmcs['clientdetails']['email'];
        $client['metadata'] = array( 'whmcs_client_id' => $whmcs['clientdetails']['userid'],
                                        'whmcs_client_url' => $client['url'] );
        $client['full_name'] = $whmcs['clientdetails']['firstname'] . ' ' . $whmcs['clientdetails']['lastname'];

        return $client;
    }

    public static function invoiceParser($whmcs)
    {
        $invoice = array();
        $invoice['amount'] = $whmcs['amount'] * 100;
        $invoice['url'] = $whmcs['returnurl'];
        $invoice['metadata'] = array( 'whmcs_invoice_id' => $whmcs['invoiceid'],
                                        'whmcs_invoice_url' => $whmcs['returnurl'] );
        $invoice['id'] = $whmcs['invoiceid'];

        return $invoice;
    }

    public static function filterID($id, $prefix)
    {
        return substr( $id, 0, strlen($prefix) ) === $prefix;
    }

    public static function formatUnixTime($unix_time)
    {
        $unix_time = $unix_time + 28800;    // add 28800 seconds to convert from UTC to Malaysian time

        return gmdate('F j, Y, g:i a', $unix_time);     // e.g. March 10, 2001, 5:16 pm. See https://www.php.net/manual/en/function.date.php for more date formats
    }

    public static function sanitizeBankName($bank_name)
    {
        $bank_name = str_replace('_', ' ', $bank_name);

        return strtoupper($bank_name);
    }
}