/*
    Last Updated: 24-FEB-21
*/

// Create and initialize a Stripe client
var publishable_key = document.getElementById('publishable_key').value;
var stripe = Stripe(publishable_key);

/*
    Description:
        1. Collect payment method details using FPX Bank Element

    References:
        1. https://stripe.com/docs/payments/fpx/accept-a-payment#collect-payment-method-details
*/
var elements = stripe.elements();

// Styling options for the FPX bank element. See https://stripe.com/docs/js/appendix/style for more options
var style = {
    base: {
        color: '#333',
        fontFamily: 'Open Sans, Verdana, Tahoma, serif',
        fontSize: '14px',
        padding: '10px 12px'
    },
};

// Create an instance of the FPX bank element
var fpx_bank = elements.create(
    'fpxBank',
    {
        style: style,
        accountHolderType: 'individual',
    }
);

// Add an instance of the FPX bank element into a <div> container
fpx_bank.mount('#fpx_bank_element');

// Enable the call to action button once the user has selected a bank
fpx_bank.on('change', function(event) {
    if (event.complete) {
        call_to_action.disabled = false;
    }
});

/*
    Description:
        1. Submit the payment to Stripe

    References:
        1. https://stripe.com/docs/payments/fpx/accept-a-payment#submit-payment
*/

// Redirect to the bank's authorization page so that the user can complete the payment
function executeFPX(event) {
    event.preventDefault();

    var client_secret = document.getElementById('client_secret').value;
    var return_url = document.getElementById('return_url').value;

    stripe.confirmFpxPayment(
        client_secret,
        {
            payment_method: {
                fpx: fpx_bank,
            },
            return_url: return_url,
        }
    ).then(function(result) {
        if (result.error) {
            var fpx_error_element = document.getElementById('fpx_error_element');
            fpx_error_element.textContent = result.error.message;
        }
    });
}

var call_to_action = document.getElementById('call_to_action');
call_to_action.addEventListener('click', executeFPX);